#!/usr/bin/perl
#
# Version 1.0.0
# Last update: 2017-05-08
#
# (c)2017 Juergen Vigna <juergen.vigna@wuerth-phoenix.com>
# Copyleft by GNU GPL
#

use lib "/var/lib/neteye/thruk/lib";
use lib "/var/lib/neteye/API/perl";

#use strict;
use warnings;
use Getopt::Long;
use Data::Dumper;

use NetEye::Backend;
use NetEye::Config;
use NetEye::Monitoring::Contact;
use NetEye::Monitoring::Group::Contact;
use NetEye::Monitoring::Template::Contact;
use NetEye::Monitoring::Connector::Contactgroup_Contact;
use NetEye::Usermanagement::User;
use NetEye::Usermanagement::Profile;

&Getopt::Long::config('auto_abbrev');

# ----------------------------------------

# Subs declaration
sub usage;
sub printd;
sub add_neteye_user;
sub add_neteye_contact;

my $ldaphost = "ldaphost";
my $ldapuser = "neteye";
my $ldappasswd = "secret";
my $ldapbasedn = "dc=domain,dc=com";
my $groupregex = "gg*-neteye-view";
my $userprofile = "User";
# BUG for the moment one cannot search Contact Templates
#my $templatename = "login";
my $templatename = 2;
my $debug = 0;

# Evaluate Command Line Parameters
my $status = GetOptions(
		        "ldaphost=s",\$ldaphost,
			"ldapuser=s",\$ldapuser,
                        "ldappasswd=s",\$ldappasswd,
			"ldapbasedn=s",\$ldapbasedn,
			"groupregex",\$groupregex,
			"userprofile",\$userprofile,
			"templatename",\$templatename,
                        "debug", \my @debug,
                        "help", \my @help,
			);

usage() if scalar @help;
usage() if ($status == 0 || ! ($ldaphost && $ldapuser && $ldappasswd && $groupregex));

$debug = scalar @debug;

use Net::LDAP;
use Data::Dumper;

my @viewgroups = ();

my $ldap = Net::LDAP->new( $ldaphost ) or die "$@";

my $mesg = $ldap->bind($ldapuser, password => $ldappasswd);

$mesg = $ldap->search(base   => $ldapbasedn,
                      filter => "(&(objectCategory=group)(cn=$groupregex))",
                      attrs  => [ "dn" ],
                     );

$mesg->code && die $mesg->error;

my $max = $mesg->count;
for ( my $i = 0 ; $i < $max ; $i++ ) {
    my $entry = $mesg->entry ( $i );
    push @viewgroups, $mesg->entry($i)->dn();
}

foreach my $group (@viewgroups) {
    printd "DN:".$group."\n";
    $mesg = $ldap->search(base   => $ldapbasedn,
                          filter => "(&(objectCategory=user)(memberOf=$group))",
                          attrs  => [ "cn", "samaccountname", "mail" ],
                         );
    if ($debug) {
        foreach my $entry ($mesg->entries) {
            Data::Dumper::Dumper($entry->attributes);
        }
    }
    my $groupname = $group;
    $groupname =~ s/.*CN=([^,]+).*/$1/gi;
    remove_all_contacts_from_group($groupname);
    my $max = $mesg->count;
    for ( my $i = 0 ; $i < $max ; $i++ ) {
        my $entry = $mesg->entry ( $i );
	my $userid = $entry->get_value("samaccountname");
	my $username = $entry->get_value("cn");
	my $usermail = $entry->get_value("mail");
	if (!defined($username)) { $username = $userid; }
	if (!defined($usermail)) { $usermail = $userid; }
	add_neteye_user($userid, $username, $usermail, $userprofile);
        add_neteye_contact($userid, $username, $usermail, $templatename, $groupname);
    }
}
$mesg = $ldap->unbind;

#
# Remove unused contactgroups
#

my %Param;
my @search;
my $contactgroup;

my $mysqlregex = $groupregex;
$mysqlregex =~ s/\*/\%/g;
$Param{ConfigObject} = new NetEye::Config();
@search = new NetEye::Monitoring::Group::Contact(%Param)->searchFor(
    CustomWhere => "contactgroups.name LIKE '$mysqlregex'",
);
if (scalar(@search)) {
    foreach my $group (@search) {
        my $gname = $group->{StoredProperties}->{name};
        my $match = grep { /$gname/ } @viewgroups;
        if (!$match) {
            printd "Delete group $gname ($match)\n";
            $group->remove();
        }
    }
}

#
# Remove unused contacts and users
#

@search = new NetEye::Monitoring::Contact(%Param)->searchFor(
    CustomWhere => "contacts.name LIKE '%'",
);
if (scalar(@search)) {
    foreach my $contact (@search) {
        my $contact_id = $contact->getId();
        my $cname = $contact->{StoredProperties}->{name};
        my @cused = new NetEye::Monitoring::Connector::Contactgroup_Contact(%Param)->searchFor(
            Properties      => {
                contact_id => $contact_id,
            },
        );
        if (!scalar(@cused)) {
            printd "Delete contact $cname ($contact_id)\n";
            $contact->remove();
            my @users = new NetEye::Usermanagement::User(%Param)->searchFor(
                Properties      => {
                    name    => $cname,
                },
            );
            if (scalar(@users)) {
                printd "Delete user $cname\n";
                $users[0]->remove();
            }
        }
    }
}

# ---------------------------------- FUNCTIONS -----------------------------------------

sub usage {
  print "handle_neteye_users - Create/Remove neteye users/view contacts starting from AD-Groups\n";
  print "\nThe following options are available:\n";
  print "   -ldaphost=text     IP or name of the (AD) LDAP-host (default: $ldaphost)\n";
  print "   -ldapuser=text     LDAP Bind Username (default: $ldapuser)\n";
  print	"   -ldappasswd=text   LDAP Bind Password (default: ********)\n";
  print	"   -ldapbasedn=text   LDAP Base DN (default: $ldapbasedn)\n";
  print	"   -groupregex=text   LDAP regexp for groups being handled (default: $groupregex)\n";
  print	"   -userprofile=text  NetEye User Profile to use (default: $userprofile)\n";
  print	"   -templatename=text NetEye Contact template to use (default: $templatename)\n";
  print "   -debug             Print Debug info\n\n";
  print "   -help              Print Debug info\n\n";
  print " Options may abbreviated!\n";
  print "\nExample: \n";
  print " $0 -ldaph=host -ldappa=pw\n ";
  print "\nCopyleft 2017, Juergen Vigna <juergen.vigna\@wuerth-phoenix.com>\n";
  print "This programm is licensed under the terms of the ";
  print "GNU General Public License\n\n";
  exit 0;
}

sub printd {
  my ($msg) = @_;
  if ($debug > 0) {
    print $msg;
  }
}

sub add_neteye_user {
    my ($userid, $username, $usermail, $userprofile) = @_;
    my %Param;
    my @search;

    if ($debug) {
        printd "USER: $userid\nNAME: $username\nMAIL: $usermail\n";
    }

    $Param{ConfigObject} = new NetEye::Config();
    @search = new NetEye::Usermanagement::Profile(%Param)->searchFor(
        Properties      => {
            name    => $userprofile,
        },
    );
    if (!scalar(@search)) {
        print "Error: Profile ($userprofile) does not exist!";
        exit 1;
    }
    my $profile_id = $search[0]->getId();

    # User has to be added
    my $User = new NetEye::Usermanagement::User(
        %Param,
        Properties => {
            name  => $userid,
            realname => $username,
            profile_id => $profile_id,
            email      => $usermail,
            protected  => 0,
            is_local   => 0,
        },
    );
    printd "User $userid created/updated!\n";
}

sub add_neteye_contact {
    my ($userid, $username, $usermail, $templatename, $groupname) = @_;
    my %Param;
    my @search;
    my $contactgroup;
    my $contact;
    my $contacttemplate_id;

    $Param{ConfigObject} = new NetEye::Config();
# BUG does not work for the moment!!!
#    @search = new NetEye::Monitoring::Template::Contact(%Param)->searchFor(
#        Properties      => {
#            name    => $templatename,
#        },
#    );
#    if (!scalar(@search)) {
#        print "ContactTemplate $templatename does not exist, refusing to create contact $userid!\n";
#	return 0;
#    }
#    $contacttemplate_id = $search[0]->getId();
    $contacttemplate_id = $templatename;

    $contactgroup = new NetEye::Monitoring::Group::Contact(
        %Param,
        Properties => {
            name  => $groupname,
            alias => $groupname,
        },
    );
    printd "Contactgroup $groupname created/updated\n";

    $contact = new NetEye::Monitoring::Contact(
        %Param,
        Properties => {
            name               => $userid,
            alias              => $username,
            email              => $usermail,
            contacttemplate_id => $contacttemplate_id,
            status             => 1,
        },
    );
    printd "Contact $userid created/updated\n";
    $contactgroup->addContact(ContactObject => $contact);

    return 1;
}

sub remove_all_contacts_from_group {
    my ($groupname) = @_;
    my %Param;
    my @search;
    my $contactgroup;

    $Param{ConfigObject} = new NetEye::Config();
    @search = new NetEye::Monitoring::Group::Contact(%Param)->searchFor(
        Properties      => {
            name    => $groupname,
        },
    );
    if (scalar(@search)) {
        $contactgroup = $search[0];
        printd "Removing all contacts from $groupname!\n";
        $contactgroup->removeAllContacts();
    } else {
        printd "Contactgroup $groupname does not exist nothing to remove!\n";
    }
}

# ---------------------------------------------------------------------

